@@ -15,10 +15,13 @@ GtkWidget *do_form_rw (GtkWidget *do_widget);
 GtkWidget *do_form_pict (GtkWidget *do_widget);
 GtkWidget *do_grid (GtkWidget *do_widget);
 GtkWidget *do_grid_rw (GtkWidget *do_widget);
+GtkWidget *do_grid_pict (GtkWidget *do_widget);
 GtkWidget *do_linked_grid_form (GtkWidget *do_widget);
 GtkWidget *do_linked_model_param (GtkWidget *do_widget);
-GtkWidget *do_graph (GtkWidget *do_widget);
 GtkWidget *do_console (GtkWidget *do_widget);
+GtkWidget *do_selector (GtkWidget *do_widget);
+GtkWidget *do_ddl_queries (GtkWidget *do_widget);
+GtkWidget *do_graph (GtkWidget *do_widget);
 
 Demo child0[] = {
   { "Read-only form", "form.c", do_form, NULL },
@@ -30,6 +33,7 @@ Demo child0[] = {
 Demo child1[] = {
   { "Read-only grid", "grid.c", do_grid, NULL },
   { "Read-write grid", "grid_rw.c", do_grid_rw, NULL },
+  { "Using the picture plugin", "grid_pict.c", do_grid_pict, NULL },
   { NULL } 
 };
 
@@ -40,7 +44,9 @@ Demo child2[] = {
 };
 
 Demo gnomedb_demos[] = {
+  { "DDL queries", "ddl_queries.c", do_ddl_queries, NULL }, 
   { "Data widgets linking", NULL, NULL, child2 }, 
+  { "Dictionary browser", "selector.c", do_selector, NULL }, 
   { "Forms", NULL, NULL, child0 }, 
   { "Graph", "graph.c", do_graph, NULL }, 
   { "Grids", NULL, NULL, child1 }, 
