@@ -22,7 +22,7 @@
 #include <stdlib.h>
 #include <glib/gi18n-lib.h>
 #include "gnome-db-data-cell-renderer-info.h"
-#include <libgnomedb/marshal.h>
+#include "marshal.h"
 #include <libgda/libgda.h>
 #include <gdk/gdkkeysyms.h>
 #include "utility.h"
