@@ -51,8 +51,9 @@ static void            gnome_db_entry_combo_set_value_orig    (GnomeDbDataEntry
 static const GValue   *gnome_db_entry_combo_get_value_orig    (GnomeDbDataEntry *de);
 static void            gnome_db_entry_combo_set_value_default (GnomeDbDataEntry *de, const GValue * value);
 static void            gnome_db_entry_combo_set_attributes    (GnomeDbDataEntry *de, guint attrs, guint mask);
-static guint           gnome_db_entry_combo_get_attributes    (GnomeDbDataEntry *de);
+static GdaValueAttribute gnome_db_entry_combo_get_attributes    (GnomeDbDataEntry *de);
 static gboolean        gnome_db_entry_combo_expand_in_layout  (GnomeDbDataEntry *de);
+static void            gnome_db_entry_combo_grab_focus        (GnomeDbDataEntry *de);
 
 /* properties */
 enum
@@ -137,6 +138,7 @@ gnome_db_entry_combo_data_entry_init (GnomeDbDataEntryIface *iface)
         iface->get_attributes = gnome_db_entry_combo_get_attributes;
         iface->get_handler = NULL;
         iface->expand_in_layout = gnome_db_entry_combo_expand_in_layout;
+	iface->grab_focus = gnome_db_entry_combo_grab_focus;
 }
 
 
@@ -155,6 +157,12 @@ gnome_db_entry_combo_class_init (GnomeDbEntryComboClass *class)
         g_object_class_install_property (object_class, PROP_SET_DEFAULT_IF_INVALID,
 					 g_param_spec_boolean ("set_default_if_invalid", NULL, NULL, FALSE,
                                                                (G_PARAM_READABLE | G_PARAM_WRITABLE)));
+
+	/* RC setting */
+#define RC_STRING  \
+	"style \"gnomedb\" { GnomeDbCombo::appears-as-list = 1 }" \
+	"class \"GtkComboBox\" style \"gnomedb\""
+	/*gtk_rc_parse_string (RC_STRING);*/
 }
 
 static void
@@ -228,23 +236,41 @@ GtkWidget *
 gnome_db_entry_combo_new (GdaParameterList *paramlist, GdaParameterListSource *source)
 {
 	GObject *obj;
-	GnomeDbEntryCombo *combo;
+
+	obj = g_object_new (GNOME_DB_TYPE_ENTRY_COMBO, NULL);
+  
+  _gnome_db_entry_combo_construct(GNOME_DB_ENTRY_COMBO (obj), paramlist, source);
+
+	return GTK_WIDGET (obj);
+}
+
+/**
+ * _gnome_db_entry_combo_construct
+ * @combo: a #GnomeDbEntryCombo object to be construced
+ * @paramlist: a #GdaParameterList object
+ * @source: a #GdaParameterListSource structure, part of @paramlist
+ *
+ * 
+ * TODO: This is just a work-around for language bindings. Ideally we would use construction
+ * properties instead.
+*/
+
+void _gnome_db_entry_combo_construct(GnomeDbEntryCombo* combo, 
+                                     GdaParameterList *paramlist, GdaParameterListSource *source)
+{
 	GSList *list;
 	GSList *values;
 	GtkWidget *entry;
 	gboolean null_possible;
 
-	g_return_val_if_fail (GDA_IS_PARAMETER_LIST (paramlist), NULL);
-	g_return_val_if_fail (source, NULL);
-	g_return_val_if_fail (g_slist_find (paramlist->sources_list, source), NULL);
-
-	obj = g_object_new (GNOME_DB_TYPE_ENTRY_COMBO, NULL);
-	combo = GNOME_DB_ENTRY_COMBO (obj);
-
-	combo->priv->paramlist = paramlist;
+	g_return_if_fail (GDA_IS_PARAMETER_LIST (paramlist));
+	g_return_if_fail (source);
+	g_return_if_fail (g_slist_find (paramlist->sources_list, source));
+  
+  combo->priv->paramlist = paramlist;
 	combo->priv->source = source;
 	gda_object_connect_destroy (GDA_OBJECT (paramlist), 
-				    G_CALLBACK (paramlist_destroyed_cb), obj);	
+				    G_CALLBACK (paramlist_destroyed_cb), G_OBJECT(combo));	
 
 	/* create the ComboNode structures, 
 	 * and use the values provided by the parameters to display the correct row */
@@ -280,8 +306,6 @@ gnome_db_entry_combo_new (GdaParameterList *paramlist, GdaParameterListSource *s
 	gnome_db_combo_set_values_ext (GNOME_DB_COMBO (entry), values, NULL);
 	g_slist_free (values);
 	gnome_db_combo_add_undef_choice (GNOME_DB_COMBO (entry), combo->priv->null_possible);
-
-	return GTK_WIDGET (obj);
 }
 
 static void
@@ -478,20 +502,18 @@ gnome_db_entry_combo_set_values (GnomeDbEntryCombo *combo, GSList *values)
 			err = TRUE;
 	}
 	else  { /* set to NULL */
-		if (combo->priv->null_possible) {
-			GSList *list;
+		GSList *list;
+		list = combo->priv->combo_nodes;
+		while (list) {
+			COMBO_NODE (list->data)->value = NULL;
+			list = g_slist_next (list);
+		}
 
+		if (combo->priv->null_possible) {
 			real_combo_block_signals (combo);
 			gtk_combo_box_set_active (GTK_COMBO_BOX (combo->priv->combo_entry), -1);
 			real_combo_unblock_signals (combo);
 
-			/* adjusting the values */
-			list = combo->priv->combo_nodes;
-			while (list) {
-				COMBO_NODE (list->data)->value = NULL;
-				list = g_slist_next (list);
-			}
-
 			combo->priv->null_forced = TRUE;
 			combo->priv->default_forced = FALSE;
 		}
@@ -611,7 +633,7 @@ gnome_db_entry_combo_set_values_orig (GnomeDbEntryCombo *combo, GSList *values)
 			
 			if (arg_value)
 				type = G_VALUE_TYPE ((GValue*) arg_value);
-			equal = (type == gda_column_get_gda_type (attrs));
+			equal = (type == gda_column_get_g_type (attrs));
 			
 			nodes = g_slist_next (nodes);
 			argptr = g_slist_next (argptr);
@@ -903,10 +925,10 @@ gnome_db_entry_combo_set_attributes (GnomeDbDataEntry *iface, guint attrs, guint
 	g_signal_emit_by_name (G_OBJECT (combo), "status_changed");
 }
 
-static guint
+static GdaValueAttribute
 gnome_db_entry_combo_get_attributes (GnomeDbDataEntry *iface)
 {
-	guint retval = 0;
+	GdaValueAttribute retval = 0;
 	GnomeDbEntryCombo *combo;
 	GSList *list;
 	GSList *list2;
@@ -1022,3 +1044,16 @@ gnome_db_entry_combo_expand_in_layout (GnomeDbDataEntry *iface)
 
 	return FALSE;
 }
+
+static void
+gnome_db_entry_combo_grab_focus (GnomeDbDataEntry *iface)
+{
+	GnomeDbEntryCombo *combo;
+
+	g_return_if_fail (iface && GNOME_DB_IS_ENTRY_COMBO (iface));
+	combo = GNOME_DB_ENTRY_COMBO (iface);
+	g_return_if_fail (combo->priv);
+
+	if (combo->priv->combo_entry)
+		gtk_widget_grab_focus (combo->priv->combo_entry);
+}
