@@ -34,7 +34,7 @@ static void gnome_db_entry_none_finalize (GObject *object);
 static GtkWidget *create_entry (GnomeDbEntryWrapper *mgwrap);
 static void       real_set_value (GnomeDbEntryWrapper *mgwrap, const GValue *value);
 static GValue  *real_get_value (GnomeDbEntryWrapper *mgwrap);
-static void       connect_signals(GnomeDbEntryWrapper *mgwrap, GCallback modif_cb, GCallback activ_cb);
+static void       connect_signals(GnomeDbEntryWrapper *mgwrap, GCallback modify_cb, GCallback activate_cb);
 static gboolean   expand_in_layout (GnomeDbEntryWrapper *mgwrap);
 
 
@@ -213,7 +213,7 @@ real_get_value (GnomeDbEntryWrapper *mgwrap)
 }
 
 static void
-connect_signals(GnomeDbEntryWrapper *mgwrap, GCallback modif_cb, GCallback activ_cb)
+connect_signals(GnomeDbEntryWrapper *mgwrap, GCallback modify_cb, GCallback activate_cb)
 {
 	GnomeDbEntryNone *entry;
 
