@@ -39,8 +39,9 @@ typedef struct _GnomeDbDataWidgetInfoPriv  GnomeDbDataWidgetInfoPriv;
 
 typedef enum 
 {
+	GNOME_DB_DATA_WIDGET_INFO_NONE = 0,
 	GNOME_DB_DATA_WIDGET_INFO_CURRENT_ROW    = 1 << 0,
-	GNOME_DB_DATA_WIDGET_INFO_ROW_MODIF_BUTTONS = 1 << 2,
+	GNOME_DB_DATA_WIDGET_INFO_ROW_MODIFY_BUTTONS = 1 << 2,
 	GNOME_DB_DATA_WIDGET_INFO_ROW_MOVE_BUTTONS = 1 << 3,
 	GNOME_DB_DATA_WIDGET_INFO_CHUNCK_CHANGE_BUTTONS = 1 << 4
 } GnomeDbDataWidgetInfoFlag;
@@ -64,7 +65,7 @@ struct _GnomeDbDataWidgetInfoClass
  */
 GType             gnome_db_data_widget_info_get_type                  (void);
 
-GtkWidget        *gnome_db_data_widget_info_new                       (GnomeDbDataWidget *data_widget, guint flags);
+GtkWidget        *gnome_db_data_widget_info_new                       (GnomeDbDataWidget *data_widget, GnomeDbDataWidgetInfoFlag flags);
 
 G_END_DECLS
 
