@@ -1,6 +1,6 @@
 /* gnome-db-grid.h
  *
- * Copyright (C) 2002 - 2005 Vivien Malerba
+ * Copyright (C) 2002 - 2006 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,22 +22,19 @@
 #define __GNOME_DB_GRID__
 
 #include <gtk/gtk.h>
-#include <libgda/gda-data-model.h>
-#include "gnome-db-dict.h"
-#include "gnome-db-enums.h"
+#include <libgda/gda-decl.h>
 
 G_BEGIN_DECLS
 
 #define GNOME_DB_TYPE_GRID          (gnome_db_grid_get_type())
 #define GNOME_DB_GRID(obj)          G_TYPE_CHECK_INSTANCE_CAST (obj, gnome_db_grid_get_type(), GnomeDbGrid)
 #define GNOME_DB_GRID_CLASS(klass)  G_TYPE_CHECK_CLASS_CAST (klass, gnome_db_grid_get_type (), GnomeDbGridClass)
-#define IS_GNOME_DB_GRID(obj)       G_TYPE_CHECK_INSTANCE_TYPE (obj, gnome_db_grid_get_type ())
+#define GNOME_DB_IS_GRID(obj)       G_TYPE_CHECK_INSTANCE_TYPE (obj, gnome_db_grid_get_type ())
 
 
 typedef struct _GnomeDbGrid      GnomeDbGrid;
 typedef struct _GnomeDbGridClass GnomeDbGridClass;
 typedef struct _GnomeDbGridPriv  GnomeDbGridPriv;
-typedef void (* GnomeDbGridForeachFunc) (GnomeDbGrid *grid, gint row, gpointer user_data);
 
 /* struct for the object's data */
 struct _GnomeDbGrid
@@ -51,30 +48,17 @@ struct _GnomeDbGrid
 struct _GnomeDbGridClass
 {
 	GtkVBoxClass       parent_class;
-
-	void             (* selection_changed) (GnomeDbGrid *grid, gboolean row_selected);
-	void             (* double_clicked)    (GnomeDbGrid *grid, gint row);
-        void             (* populate_popup)    (GnomeDbGrid *grid, GtkMenu *menu);
 };
 
 /* 
  * Generic widget's methods 
  */
-GType             gnome_db_grid_get_type                  (void);
+GType             gnome_db_grid_get_type            (void);
 
-GtkWidget        *gnome_db_grid_new                       (void);
-GtkWidget        *gnome_db_grid_new_with_gda_model        (GnomeDbDict *dict, GdaDataModel *model);
-GtkWidget        *gnome_db_grid_new_with_select_query     (GnomeDbQuery *query, GnomeDbTarget *modified);
+GtkWidget        *gnome_db_grid_new                 (GdaDataModel *model);
+GList            *gnome_db_grid_get_selection       (GnomeDbGrid *grid);
+void              gnome_db_grid_set_sample_size     (GnomeDbGrid *grid, gint sample_size);
 
-void              gnome_db_grid_set_sample_size           (GnomeDbGrid *grid, gint sample_size);
-void              gnome_db_grid_set_sample_start          (GnomeDbGrid *grid, gint sample_start);
-
-void              gnome_db_grid_get_selection_mode        (GnomeDbGrid *grid, GtkSelectionMode mode);
-GList            *gnome_db_grid_get_selection             (GnomeDbGrid *grid);
-void              gnome_db_grid_select_all                (GnomeDbGrid *grid);
-void              gnome_db_grid_unselect_all              (GnomeDbGrid *grid);
-void              gnome_db_grid_foreach_selected          (GnomeDbGrid *grid,
-							   GnomeDbGridForeachFunc foreach_func, gpointer user_data);
 G_END_DECLS
 
 #endif
