@@ -27,10 +27,9 @@
 #include <glade/glade-init.h>
 #endif
 #include <libgda/libgda.h>
-#ifdef BUILD_WITH_GNOME
-#include <libgnomeui/gnome-ui-init.h>
-#endif
+#ifdef HAVE_GCONF
 #include <gconf/gconf-client.h>
+#endif
 #include <libgda/graph/gda-dict-reg-graphs.h>
 
 /* plugins list */
@@ -61,23 +60,18 @@ gnome_db_init (const gchar *app_id, const gchar *version, gint nargs, gchar *arg
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 
 	gda_init (app_id, version, nargs, args);
-
-#ifdef BUILD_WITH_GNOME
-	gnome_program_init (app_id, version, LIBGNOMEUI_MODULE, nargs, args,
-			    GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
-
-#else
 	gtk_init (&nargs, &args);
-#endif
 
 #ifdef HAVE_LIBGLADE
 	glade_init ();
 #endif
 	gnome_db_stock_init ();
 
+#ifdef HAVE_GCONF
 	/* init GConf */
 	if (!gconf_is_initialized ())
 		gconf_init (0, NULL, NULL);
+#endif
 
 	/* make sure all the GdaDict object register GdaGraph */
 	gda_dict_class_always_register (gda_graphs_get_register);
