@@ -73,6 +73,7 @@ gnome_db_util_new_data_entry (GdaDict *dict, GType type, const gchar *plugin_nam
 {
 	GdaDataHandler *dh;
 	GnomeDbDataEntry *entry = NULL;
+	gchar *spec_options = NULL;
 
 	if (!gnome_db_plugins_hash)
 		gnome_db_plugins_hash = init_plugins_hash ();
@@ -87,8 +88,10 @@ gnome_db_util_new_data_entry (GdaDict *dict, GType type, const gchar *plugin_nam
 		for (ptr = plugin; *ptr && (*ptr != ':'); ptr++);
 		*ptr = 0;
 		ptr++;
-		if (ptr < plugin + strlen (plugin_name))
+		if (ptr < plugin + strlen (plugin_name)) {
 			options = ptr;
+			spec_options = g_strdup (options);
+		}
 		
 		plugin_struct = g_hash_table_lookup (gnome_db_plugins_hash, plugin);
 		if (plugin_struct && plugin_struct->entry_create_func) 
@@ -113,7 +116,7 @@ gnome_db_util_new_data_entry (GdaDict *dict, GType type, const gchar *plugin_nam
 			 (type == G_TYPE_UCHAR) ||
 			 (type == G_TYPE_ULONG) ||
 			 (type == G_TYPE_UINT))
-			entry = (GnomeDbDataEntry *) gnome_db_entry_string_new (dh, type);
+			entry = (GnomeDbDataEntry *) gnome_db_entry_string_new (dh, type, spec_options);
 		else if (type == G_TYPE_BOOLEAN)
 			entry = (GnomeDbDataEntry *) gnome_db_entry_boolean_new (dh, G_TYPE_BOOLEAN);
 		else if	((type == GDA_TYPE_GEOMETRIC_POINT) ||
@@ -128,9 +131,10 @@ gnome_db_util_new_data_entry (GdaDict *dict, GType type, const gchar *plugin_nam
 		else if (type == G_TYPE_DATE)
 			entry = (GnomeDbDataEntry *) gnome_db_entry_date_new (dh);
 		else
-			entry = (GnomeDbDataEntry *) gnome_db_entry_string_new (dh, type);
+			entry = (GnomeDbDataEntry *) gnome_db_entry_string_new (dh, type, spec_options);
 	}
 
+	g_free (spec_options);
 	return entry;
 }
 
@@ -181,11 +185,11 @@ gnome_db_util_new_cell_renderer (GdaDict *dict, GType type, const gchar *plugin_
 
 	if (!cell) {
 		if (type == GDA_TYPE_NULL)
-			cell = gnome_db_data_cell_renderer_textual_new (NULL, GDA_TYPE_NULL);
+			cell = gnome_db_data_cell_renderer_textual_new (NULL, GDA_TYPE_NULL, NULL);
 		else if (type == G_TYPE_BOOLEAN)
 			cell = gnome_db_data_cell_renderer_boolean_new (dh, G_TYPE_BOOLEAN);
 		else
-			cell = gnome_db_data_cell_renderer_textual_new (dh, type);
+			cell = gnome_db_data_cell_renderer_textual_new (dh, type, NULL);
 	}
 
 	return cell;
@@ -206,6 +210,7 @@ init_plugins_hash (void)
 {
 	GHashTable *hash;
 	GnomeDbPlugin *plugin;
+	gchar *file;
 
 	hash = g_hash_table_new (g_str_hash, g_str_equal); /* key strings are not handled in the hash table */
 
@@ -233,6 +238,61 @@ init_plugins_hash (void)
 	plugin->cell_create_func =cell_boolean_create_func;
 	g_hash_table_insert (hash, plugin->plugin_name, plugin);
        
+	plugin = g_new0 (GnomeDbPlugin, 1);
+	plugin->plugin_name = "string";
+	plugin->plugin_descr = "String entry";
+	plugin->plugin_file = NULL;
+	plugin->nb_g_types = 1;
+	plugin->valid_g_types = g_new (GType, plugin->nb_g_types);
+	plugin->valid_g_types [0] = G_TYPE_STRING;
+	plugin->options_xml_spec = NULL;
+	plugin->entry_create_func = entry_string_create_func;
+	plugin->cell_create_func = cell_textual_create_func;
+	g_hash_table_insert (hash, plugin->plugin_name, plugin);
+	file = g_build_filename (LIBGNOMEDB_DATADIR, "gnome-db-entry-string-string.xml", NULL);
+	if (! g_file_test (file, G_FILE_TEST_EXISTS)) {
+		g_message ("Could not find file '%s': '%s' data entry will not report any possible option",
+				   file, plugin->plugin_name);
+        }
+	else {
+		gsize len;
+		g_file_get_contents (file, &(plugin->options_xml_spec), &len, NULL);
+	}
+	g_free (file);
+
+	plugin = g_new0 (GnomeDbPlugin, 1);
+	plugin->plugin_name = "number";
+	plugin->plugin_descr = "Numeric entry";
+	plugin->plugin_file = NULL;
+	plugin->nb_g_types = 12;
+	plugin->valid_g_types = g_new (GType, plugin->nb_g_types);
+	plugin->valid_g_types [0] = G_TYPE_INT64;
+	plugin->valid_g_types [1] = G_TYPE_UINT64;
+	plugin->valid_g_types [2] = G_TYPE_DOUBLE;
+	plugin->valid_g_types [3] = G_TYPE_INT;
+	plugin->valid_g_types [4] = GDA_TYPE_NUMERIC;
+	plugin->valid_g_types [5] = G_TYPE_FLOAT;
+	plugin->valid_g_types [6] = GDA_TYPE_SHORT;
+	plugin->valid_g_types [7] = GDA_TYPE_USHORT;
+	plugin->valid_g_types [8] = G_TYPE_CHAR;
+	plugin->valid_g_types [9] = G_TYPE_UCHAR;
+	plugin->valid_g_types [10] = G_TYPE_ULONG;
+	plugin->valid_g_types [11] = G_TYPE_UINT;
+	plugin->options_xml_spec = NULL;
+	plugin->entry_create_func = entry_string_create_func;
+	plugin->cell_create_func = cell_textual_create_func;
+	g_hash_table_insert (hash, plugin->plugin_name, plugin);
+	file = g_build_filename (LIBGNOMEDB_DATADIR, "gnome-db-entry-string-number.xml", NULL);
+	if (! g_file_test (file, G_FILE_TEST_EXISTS)) {
+		g_message ("Could not find file '%s': '%s' data entry will not report any possible option",
+				   file, plugin->plugin_name);
+        }
+	else {
+		gsize len;
+		g_file_get_contents (file, &(plugin->options_xml_spec), &len, NULL);
+	}
+	g_free (file);
+
 	plugin = g_new0 (GnomeDbPlugin, 1);
 	plugin->plugin_name = "textual";
 	plugin->plugin_descr = "Textual entry";
@@ -361,7 +421,7 @@ init_plugins_hash (void)
 static GnomeDbDataEntry *
 entry_none_create_func (GdaDataHandler *handler, GType type, const gchar *options)
 {
-	return (GnomeDbDataEntry *) gnome_db_entry_none_new (GDA_TYPE_NULL);
+	return (GnomeDbDataEntry *) gnome_db_entry_none_new (type);
 }
 
 static GnomeDbDataEntry *
@@ -373,7 +433,7 @@ entry_boolean_create_func (GdaDataHandler *handler, GType type, const gchar *opt
 static GnomeDbDataEntry *
 entry_string_create_func (GdaDataHandler *handler, GType type, const gchar *options)
 {
-	return (GnomeDbDataEntry *) gnome_db_entry_string_new (handler, type);
+	return (GnomeDbDataEntry *) gnome_db_entry_string_new (handler, type, options);
 }
 
 static GnomeDbDataEntry *
@@ -397,7 +457,7 @@ entry_date_create_func (GdaDataHandler *handler, GType type, const gchar *option
 static GtkCellRenderer *
 cell_textual_create_func (GdaDataHandler *handler, GType type, const gchar *options)
 {
-	return gnome_db_data_cell_renderer_textual_new (handler, type);
+	return gnome_db_data_cell_renderer_textual_new (handler, type, options);
 }
 
 static GtkCellRenderer *
